
local addon = CreateFrame("Frame")
_G.SimpleTankFrames = addon
local allFrames = {}

local backdrop = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16}
local function initCommon(frame)
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(0, 0, 0, .5)
	frame:SetWidth(120)
	frame:SetHeight(22)

	local bar = CreateFrame("StatusBar", nil, frame)
	bar:SetPoint("TOPLEFT", frame, 4, -3)
	bar:SetPoint("BOTTOMRIGHT", frame, -4, 3)
	bar:SetStatusBarTexture("Interface\\AddOns\\JustTimer\\statusbar")
	frame.hp = bar

	local tx = bar:CreateTexture(nil, "BORDER")
	tx:SetVertexColor(1, 0, 0, 0.5)
	tx:SetTexture("Interface\\AddOns\\JustTimer\\statusbar")
	tx:SetVertexColor(.5, .5, .5, .5)
	tx:SetAllPoints(bar)
	frame.bg = tx

	local value = bar:CreateFontString(nil, "OVERLAY")
	value:SetFontObject(GameFontHighlightSmall)
	value:SetJustifyH("RIGHT")
	value:SetWidth(32)
	value:SetHeight(14)
	value:SetPoint("RIGHT", bar, -8)
	frame.value = value

	local icon = bar:CreateTexture(nil, "OVERLAY")
	icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	icon:SetWidth(14)
	icon:SetHeight(14)
	icon:SetPoint("LEFT", bar, 8)
	icon:Hide()
	frame.icon = icon

	local name = bar:CreateFontString(nil, "OVERLAY")
	name:SetFontObject(GameFontHighlightSmall)
	name:SetJustifyH("CENTER")
	name:SetPoint("LEFT", bar, 22)
	name:SetPoint("RIGHT", value, -4)
	frame.name = name
end

local function icon(self, unit)
	local unit = unit or self.unit
	if not unit then return end
	local index = GetRaidTargetIndex(unit)
	if index then
		SetRaidTargetIconTexture(self.icon, index)
		self.icon:Show()
	else
		self.icon:Hide()
	end
end

local colors = { disconnected = {.6, .6, .6}, class = {}, reaction = {} }
for class, color in next, RAID_CLASS_COLORS do colors.class[class] = {color.r, color.g, color.b} end
for class, color in next, FACTION_BAR_COLORS do colors.reaction[class] = {color.r, color.g, color.b} end
local function hp(self, unit)
	if not unit or self.unit ~= unit then return end
	local unit = unit or self.unit
	local min, max = UnitHealth(unit), UnitHealthMax(unit)
	self.hp:SetMinMaxValues(0, max)
	local t = nil
	if not UnitIsConnected(unit) then
		self.hp:SetValue(max)
		self.hp:SetAlpha(0.5)
		self.value:SetText()
		t = colors.disconnected
	else
		self.hp:SetAlpha(1)
		self.hp:SetValue(min)
		local percent = max == 0 and 0 or math.floor(min / max * 100 + 0.5)
		self.value:SetFormattedText("%d%%", percent)
		if UnitIsPlayer(unit) then
			t = colors.class[(select(2, UnitClass(unit)))]
		elseif UnitReaction(unit, "player") then
			t = colors.reaction[UnitReaction(unit, "player")]
		end
	end
	if not t then return end
	self.hp:SetStatusBarColor(unpack(t))
end

local function name(self, unit)
	if not unit or self.unit ~= unit then return end
	local unit = unit or self.unit
	self.name:SetText(UnitName(self.unit))
end

local function fullUpdate(self)
	local unit = self.unit
	if not UnitExists(unit) then return end
	hp(self, unit)
	name(self, unit)
	icon(self, unit)
end

local eventFuncs = {
	PLAYER_ENTERING_WORLD = fullUpdate,
	UNIT_NAME_UPDATE = name,
	UNIT_MAXHEALTH = hp,
	UNIT_HEALTH = hp,
	RAID_TARGET_UPDATE = icon,
}
local function onEvent(self, event, unit) if eventFuncs[event] then eventFuncs[event](self, unit) end end

local function attributeChanged(self, name, value)
	if name ~= "unit" or not value then return end
	if self.unit and self.unit == value then return end

	local child = SecureButton_GetModifiedUnit(self.target)
	self.target.unit = child
	fullUpdate(self.target)

	self.unit = SecureButton_GetModifiedUnit(self)
	--self.id = value:match("^.-(%d+)")
	fullUpdate(self)
end

local function onUpdate(self, elapsed)
	if not self.unit then return end
	self.timer = self.timer + elapsed
	if self.timer >= 0.3 then -- Health updates only happen every 0.3sec I think
		fullUpdate(self)
		self.timer = 0
	end
end

local function initTankPair(tank)
	local target = tank:GetChildren()
	tank.target = target

	initCommon(tank)
	initCommon(target)

	tank:RegisterEvent("UNIT_MAXHEALTH")
	tank:RegisterEvent("UNIT_HEALTH")
	tank:RegisterEvent("RAID_TARGET_UPDATE")
	tank:SetScript("OnEvent", onEvent)
	tank:SetScript("OnAttributeChanged", attributeChanged)

	target.timer = 0
	target:SetScript("OnUpdate", onUpdate)

	allFrames[#allFrames + 1] = tank
	allFrames[#allFrames + 1] = target
end

local function createFrames()
	local f = CreateFrame("Frame", nil, UIParent)
	f:SetMovable(true)
	f:SetWidth(150)
	f:SetHeight(13)
	f:SetPoint("CENTER", UIParent)
	f:SetAlpha(1)
	f:SetScale(1)
	f:EnableMouse(true)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", function(self) if not InCombatLockdown() then self:StartMoving() end end)
	f:SetScript("OnDragStop", function(self) if not InCombatLockdown() then self:StopMovingOrSizing() end end)

	local text = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	text:SetText("MainTank")
	text:SetAllPoints(f)

	local header = CreateFrame("Frame", "SimpleTankFrames", f, "SecureRaidGroupHeaderTemplate")
	addon.header = header
	header:SetFrameStrata("BACKGROUND")
	header:SetWidth(150)
	header:SetHeight(13)
	header:SetPoint("TOP", f, "BOTTOM")
	header:SetAttribute("template", "SimpleTankTemplate")
	header:SetAttribute("point", "BOTTOM")
	header:SetAttribute("sortDir", "DESC")
	header:SetAttribute("showRaid", true)
	header:SetAttribute("initial-unitWatch", true)
	header:SetAttribute("yOffset", 1)
	header.initialConfigFunction = initTankPair
	header:Show()
	f:Show()
end

local function updateTanks(event, tanks)
	addon.header:SetAttribute("nameList", table.concat(tanks, ","))
end

addon:SetScript("OnEvent", function(self, event)
	if event == "UNIT_TARGET" then
		for i, frame in next, allFrames do
			if UnitIsUnit(frame.unit, "target") then
				frame:SetBackdropColor(1, .84, 0, 1)
			else
				frame:SetBackdropColor(0, 0, 0, .5)
			end
		end
	else
		createFrames()
		oRA3.RegisterCallback(self, "OnTanksUpdated", updateTanks)
	end
end)

addon:RegisterEvent("VARIABLES_LOADED")
addon:RegisterEvent("UNIT_TARGET")



